@@ -30,15 +30,8 @@ class Gui:
 	def __init__(self, backupsets):
 		self.tmpdir = "/tmp"
 		
-		try:
-		    self.widgets = gtk.glade.XML("%s/gui.glade" % os.path.dirname(__file__))
-		except RuntimeError, e:
-		    dlg = gtk.MessageDialog(
-			None,
-			gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE,
-			_("Unable to start %(progname)s:\n%(error)s") % {'progname':version.APPNAME, 'error':e})
-		    dlg.connect("response", lambda w: gtk.main_quit())
-		    sys.exit(1)
+		self.widgets = gtk.glade.XML("%s/gui.glade" % os.path.dirname(__file__))
+		self.win_main = self.widgets.get_widget("window_main")
 
 		self.widgets.signal_autoconnect(self)
 		self.widgets.get_widget('set_destination').set_model(
@@ -49,7 +42,9 @@ class Gui:
 		self.backupsets.add_change_hook(self.refresh_set_list)
 		self.seteditor = SetEditor(self.backupsets)
 		self.statuswin = StatusWindow()
-		self.dialogs = dialogs.Dialogs(self.widgets.get_widget('window_main'))
+		self.cdprogress = CDProgressWindow(self.win_main)
+		self.cdprogress.set_response_cb(self.cdprogress_response_cb)
+		self.dialogs = dialogs.Dialogs(self.win_main)
 
 		# populate the backup set list on the main window
 		self.widgets.get_widget('treeview_backup_sets').set_model(
@@ -71,7 +66,7 @@ class Gui:
 			_("To begin, you need to create a new backup set. Click the 'New' button to get started."))
 
 		# other widget initialisation
-		self.widgets.get_widget('window_main').set_title(_("File Backup Manager"))
+		self.win_main.set_title(_("File Backup Manager"))
 		self.widgets.get_widget('lbl_restore_help').set_text(
 		    self.widgets.get_widget('lbl_restore_help').get_text().
 		    replace(r'%s', version.APPPATH))
@@ -165,7 +160,7 @@ class Gui:
 			self.dialogs.showerror(_("You cannot delete this set."))
 			return True
 			
-		dlg = gtk.MessageDialog(self.widgets.get_widget('window_main'),
+		dlg = gtk.MessageDialog(self.win_main,
 			gtk.DIALOG_MODAL, gtk.MESSAGE_WARNING, gtk.BUTTONS_YES_NO,
 			_("Are you sure you wish to delete the backup set '%s'?\n"
                           "Deleting this backup set will not delete any previous backups, "
@@ -177,7 +172,6 @@ class Gui:
 	def on_delete_set_dialog_response(self, widget, response, set):
 		if response == gtk.RESPONSE_YES:
 			set.delete()
-			print "Removing..."
 			self.backupsets.remove(set)
 			self.widgets.get_widget('backup_controls').set_sensitive(False)
 			self.widgets.get_widget('button_delete_set').set_sensitive(False)
@@ -217,7 +211,7 @@ class Gui:
 		self.widgets.get_widget('lbl_status').set_text(_("Starting backup..."))
 		self.widgets.get_widget('notebook1').set_sensitive(False)
 		self.widgets.get_widget('menubar1').set_sensitive(False)
-		self.widgets.get_widget('window_main').window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
+		self.win_main.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
 		while gtk.events_pending():
 			gtk.main_iteration()
 		try:
@@ -230,7 +224,7 @@ class Gui:
 		finally:
 			self.widgets.get_widget('notebook1').set_sensitive(True)
 			self.widgets.get_widget('menubar1').set_sensitive(True)
-			self.widgets.get_widget('window_main').window.set_cursor(None)	
+			self.win_main.window.set_cursor(None)	
 
 	def show_progress(self, progress=0):
 		self.widgets.get_widget('progressbar1').set_fraction(progress/100)
@@ -301,10 +295,15 @@ class Gui:
 		isomaker = Mkisofs()
 		isomaker.set_progress_hook(self.show_progress)
 		isomaker.create_iso(dest_path, isopath)
-		if isomaker.get_retval() != 0:
+		isoret = isomaker.get_retval()
+		if isoret != 0:
+			errmsg = os.strerror(isoret)
 			self.widgets.get_widget('lbl_status').set_text(_("Ready."))
 			self.widgets.get_widget('progressbar1').set_fraction(0)
-			self.statuswin.addmsg(_("Backup failed; could not create CD image %s\n") % isopath)
+			msg = _("Backup failed; could not create CD image %(filename)s: %(error)s\n") %\
+					{'filename':isopath, 'error':errmsg}
+			self.statuswin.addmsg(msg)
+			self.dialogs.showerror(msg)
 			return True	
 
 		# ISO is done, let's write that bad boy...
@@ -324,12 +323,15 @@ class Gui:
 		cdburner = nautilusburn.Recorder()
 		track = nautilusburn.DataTrack()
 		track.filename = isopath
-		cdburner.connect('progress-changed', self.on_cdburn_progress_changed)
-		self.widgets.get_widget('window_progress').set_transient_for(self.widgets.get_widget('window_main'))
-		self.widgets.get_widget('window_progress').show()
-		burn_result = cdburner.write_tracks(drive, [track], drive.get_max_speed_write(), nautilusburn.RECORDER_WRITE_EJECT)
+		cdburner.connect('progress-changed', self.cdprogress.on_cdburn_progress_changed)
+		self.cdprogress.show()
+		burn_result = cdburner.write_tracks(drive, [track],
+				drive.get_max_speed_write(),
+				nautilusburn.RECORDER_WRITE_EJECT)
 		if burn_result != nautilusburn.RECORDER_RESULT_FINISHED:
-			self.on_window_progress_response(gtk.RESPONSE_OK)
+			print burn_result
+			self.cdprogress.hide()
+			self.cdprogress.reset()
 			self.dialogs.showerror(_("An error occurred while burning the CD."))
 			self.widgets.get_widget('lbl_status').set_text("Ready.")
 			self.widgets.get_widget('progressbar1').set_fraction(0)
@@ -570,12 +572,8 @@ class Gui:
 		self.statuswin.addmsg(_("Backup source analysis complete."))
 		return (True, filecount)
 
-	def on_window_progress_response(self, widget, response=None):
-		self.widgets.get_widget('window_progress').hide()
-		self.widgets.get_widget('prog_button').set_sensitive(False)
-		self.widgets.get_widget('prog_status').set_markup("<i>"+_("Waiting for CD...")+"</i>")
-		self.widgets.get_widget('prog_bar').set_fraction(0)
-		self.widgets.get_widget('prog_bar').set_text("")
+	def cdprogress_response_cb(self, response):
+		self.cdprogress.reset()
 	
 	def on_backup_local_dest_button_clicked(self, event):
 		filechooser = FileChooser()
@@ -862,21 +860,6 @@ class Gui:
 	##########################
 	##     MISC HANDLERS    ##
 	##########################
-	def on_cdburn_progress_changed(self, recorder, fract, somethingelse):
-		if fract > 0:
-			self.widgets.get_widget('prog_status').set_markup("<i>"+_("Burning CD...")+"</i>")
-			self.widgets.get_widget('prog_bar').set_fraction(fract)
-			self.widgets.get_widget('prog_bar').set_text("%d%%" % (fract*100))
-		else:
-			self.widgets.get_widget('prog_status').set_markup("<i>"+_("Waiting for CD...")+"</i>")
-			self.widgets.get_widget('prog_bar').set_fraction(0)
-			self.widgets.get_widget('prog_bar').set_text("0%")
-
-		if fract == 1:
-			self.widgets.get_widget('prog_button').set_sensitive(True)
-			self.widgets.get_widget('prog_status').set_markup("<i>"+_("Finished.")+"</i>")
-		gtk.main_iteration()
-	
 	def on_recursive_copy_step(self, file):
 		self.current_count += 1.0
 		self.widgets.get_widget('progressbar1').set_fraction(self.current_count/self.total_count)
@@ -927,7 +910,7 @@ class Gui:
 				self.widgets.get_widget('cmb_backup_burner').set_active(0)
 				self.widgets.get_widget('cmb_backup_burner').set_sensitive(True)
 			
-	def refresh_set_list(self):
+	def refresh_set_list(self, selected=None):
 		treeview = self.widgets.get_widget('treeview_backup_sets')
 		setlist = treeview.get_model()
 		setlist.clear()
@@ -940,6 +923,11 @@ class Gui:
 				set_type = gtk.STOCK_DIRECTORY
 			setlist.append([treeview.render_icon(set_type, gtk.ICON_SIZE_MENU, "TreeView"), 
 							bset.name, bset])
+		if selected is not None:
+			self.select_set(selected.name)
+		else:
+			self.widgets.get_widget('button_do_backup').set_sensitive(False)
+
 		
 	def select_set(self, setname):
 		"""Iterates through the set list and places the sets treeview cursor
@@ -967,3 +955,74 @@ class Gui:
 
 		gtk.main_quit()
 
+
+
+class CDProgressWindow():
+	"""
+	The CD burning progress window.
+	"""
+	def __init__(self, parent):
+		"""
+		Load the GUI widgets and connect signal handlers. The parent
+		argument is the parent window of this CDProgressWindow.
+		"""
+		gladef = "%s/progress_window.glade" % os.path.dirname(__file__)
+		self.widgets = gtk.glade.XML(gladef)
+		self.widgets.signal_autoconnect(self)
+		self.widgets.get_widget("window_progress").set_transient_for(parent)
+	
+	def show(self):
+		"""
+		Makes the window visible.
+		"""
+		self.widgets.get_widget("window_progress").show()
+
+	def hide(self):
+		"""
+		Hides this CD progress window
+		"""
+		self.widgets.get_widget("window_progress").hide()
+	
+	def reset(self):
+		"""
+		Reset the GUI widgets to their starting states.
+		"""
+		self.widgets.get_widget('prog_button').set_sensitive(False)
+		self.widgets.get_widget('prog_status').set_markup("<i>"+_("Waiting for CD...")+"</i>")
+		self.widgets.get_widget('prog_bar').set_fraction(0)
+		self.widgets.get_widget('prog_bar').set_text("")
+
+	def set_response_cb(self, cb):
+		"""
+		Set a callback function, which must accept one argument, which
+		will be called when a button is clicked by the user.
+		"""
+		self.response_cb = cb
+
+	def on_cdburn_progress_changed(self, recorder, fract, somethingelse):
+		"""
+		Callback function to accept the progress reported by
+		the CD burner.
+		"""
+		if fract > 0:
+			self.widgets.get_widget('prog_status').set_markup("<i>"+_("Burning CD...")+"</i>")
+			self.widgets.get_widget('prog_bar').set_fraction(fract)
+			self.widgets.get_widget('prog_bar').set_text("%d%%" % (fract*100))
+		else:
+			self.widgets.get_widget('prog_status').set_markup("<i>"+_("Waiting for CD...")+"</i>")
+			self.widgets.get_widget('prog_bar').set_fraction(0)
+			self.widgets.get_widget('prog_bar').set_text("0%")
+
+		if fract == 1:
+			self.widgets.get_widget('prog_button').set_sensitive(True)
+			self.widgets.get_widget('prog_status').set_markup("<i>"+_("Finished.")+"</i>")
+		gtk.main_iteration()
+
+	def on_window_progress_response(self, widget, response=None):
+		"""
+		A callback function to be attached to the nautilusburn progress
+		signal.
+		"""
+		self.hide()
+		if callable(self.response_cb):
+			self.response_cb(response)
