@@ -312,7 +312,7 @@ class SetEditor:
             if buset not in self.backupsets:
                 self.backupsets.add(buset)
             if callable(self.exitnotify):
-                self.exitnotify()
+                self.exitnotify(buset)
             self.druid_cleanup()
         except OSError, e:
             self.widgets.get_widget('druid_page_finish').set_text(
